#include <Arduino.h>
#include "Automaton.h"
#include "TeensyStep.h"

bool debug = true ;

//stepper motor
int enablePin = 22 ;
Stepper motor(15, 14);      
RotateControl controller;  
int stepper_maxSpeed = 5000 ;
int stepper_maxAcc = 10000;

int STEPS_PER_TURN = 200 ;
int MSTEPPING = 1 ;
float MOTOR_RATIO = 99.05 ;
float BELT_RATIO = 5.217391304 ;
float STEPS_2_ROT = STEPS_PER_TURN * MSTEPPING * MOTOR_RATIO * BELT_RATIO ;


int stepper_speeds[4] = {8000, 4000, 1000, 100 } ;



//Pins declaration
int btn_run_pin = 7 ;
Atm_bit toggle;
int btn_speed_pin[4] = {9, 10, 11, 12} ;
int led_run_pin = 6 ;


//Automaton
Atm_button btn_run;
Atm_button btn_speed[4];
int current_speed ;
Atm_led led_run;


///////////////   CALLBACKS   //////////////////

void button_change( int idx, int v, int up ){
  Serial.print("Speed ");
  Serial.print(idx);
  Serial.print(" selected : ");
  Serial.println(stepper_speeds[idx]);
  current_speed = stepper_speeds[idx] ;
  motor.setMaxSpeed(current_speed);
}


///////////////   SETUP   //////////////////

void setup()
{
  Serial.begin(115200);
  Serial.println("Started");

  Serial.print("One turn is ");
  Serial.print(STEPS_2_ROT);
  Serial.println(" steps");
  Serial.print("microstepping ");
  Serial.println(MSTEPPING);


  //Automaton setup
  btn_run.begin( btn_run_pin ) 
    // .onPress( [] ( int idx, int v, int up ) {
    //  Serial.println("toggle run");
    //  led_run.trigger(led_run.EVT_TOGGLE);
    // } )
    .onPress( toggle, toggle.EVT_TOGGLE );
  
  led_run.begin(led_run_pin);

  toggle.begin() // Bit machine stores the toggle state
    .onChange( true, led_run, led_run.EVT_ON ) // And turns the led on and off
    .onChange( false, led_run, led_run.EVT_OFF );;

  
  for(int i=0; i<4; i++)  {
      btn_speed[i].begin(btn_speed_pin[i])
      .onPress(button_change, i);
    }


  //Automaton.debug
  if(debug){
    btn_run.trace(Serial);
    led_run.trace(Serial);
    for(int i=0; i<4; i++)  {
      btn_speed[i].trace(Serial);
    }
  }

  //Motor setup
  pinMode(enablePin, OUTPUT);
  digitalWrite(enablePin, LOW);
  motor.setMaxSpeed(8000);
  motor.setAcceleration(1000);
}

/////////////////////   LOOP    /////////////////////

void loop() 
{
  automaton.run();
  controller.rotateAsync(motor);
}